import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:workflow/model/response_bean.dart';
import 'package:workflow/model/simple_device.dart';
import 'package:workflow/utils/net_util.dart';
import 'package:workflow/utils/sp_util.dart';
import '../utils/sp_util.dart';
import 'log_utils.dart';
import 'package:dio/dio.dart';

const noDevicesTips = 'No devices detected.\n\n'
    "Run 'flutter emulators' to list and start any available device emulators.\n\n"
    'Or, if you expected your device to be detected, please run "flutter doctor" to diagnose '
    'potential issues, or visit https://flutter.dev/setup/ for troubleshooting tips.';

/// 设备管理client
class DeviceManager extends StatefulWidget {
  @override
  _DeviceManagerState createState() => _DeviceManagerState();

  /// 获取当前连接的设备
  static Future<SimpleDevice> currentDevice() async {
    var jsonString = await SpUtil.getString("current_device");
    if (jsonString != null && jsonString.isNotEmpty) {
      Map map = jsonDecode(jsonString);
      return SimpleDevice.fromJson(map);
    }
    return SimpleDevice.empty;
  }
}

class _DeviceManagerState extends State<DeviceManager> {
  List<SimpleDevice> list = [];

  SimpleDevice currentDevice;

  bool isLoading = false;

  bool isFilterWebDevices = false;

  @override
  void initState() {
    super.initState();
    _fetchDeviceList();
    checkWorkspace().then((result) {
      var skipUpdate = result[0];
      var path = result[1];
      if (skipUpdate) {
        return;
      }

      var bar = SnackBar(
        content: Text('检测到目录：$path'),
        action: SnackBarAction(
          label: "设为工作空间",
          textColor: Colors.white,
          onPressed: () {
            SpUtil.setString(tPathSpKey, path)
                .then((onValue) => debugPrint('path updated'));
          },
        ),
      );
      Scaffold.of(context).showSnackBar(bar);
    });
  }

  Future<List> checkWorkspace() async {
    var response = await NetUtils.get('/api/path', withCode: true);
    var path = '';
    if (response is ResponseBean) {
      if (response.code == 1) {
        path = response.code == 0 ? (response.data as String) : '';
      }
    } else if (response is Response) {
      path = response.toString();
    }

    if (path == null || path == '') {
      // unable to get path
      return [true, ''];
    }
    var current = await SpUtil.getString(tPathSpKey);
    return [path == current, path];
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.center,
      child: Column(
        children: <Widget>[
          SizedBox(
            height: 50,
          ),
          Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              _refreshBtn(_fetchDeviceList),
              _filterCheckBox(),
            ],
          ),
          SizedBox(
            height: 20,
          ),
          list.isNotEmpty
              ? Column(
                  children: <Widget>[..._generateDeviceList(list)],
                )
              : Padding(
                  padding: const EdgeInsets.all(15),
                  child: Text(
                    !isLoading ? noDevicesTips : "",
                    style: TextStyle(color: Colors.red, fontSize: 20),
                  ),
                ),
        ],
      ),
    );
  }

  List<Widget> _generateDeviceList(List<SimpleDevice> devices) {
    if (isFilterWebDevices) {
      devices = devices.where((d) {
        return "web" != d.platform;
      }).toList();
    }
    return List<Widget>.generate(devices.length, (i) {
      var id = currentDevice != null ? currentDevice.deviceId : "";
      var isSelectedOne = devices[i].deviceId == id;
      return DeviceItem(
        device: devices[i],
        isSelectedOne: isSelectedOne,
        click: (d) {
          setState(() {
            currentDevice = d;
            String jsonString = jsonEncode(d.toJson());
            SpUtil.setString("current_device", jsonString);
          });
        },
      );
    }).toList();
  }

  /// 刷新按钮
  Widget _refreshBtn(Function onClick) {
    return GestureDetector(
      onTap: onClick,
      child: Container(
        color: Colors.green,
        height: 50,
        width: 150,
        margin: EdgeInsets.symmetric(horizontal: 15),
        padding: EdgeInsets.symmetric(horizontal: 15),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Text(
              isLoading ? "加载中..." : "刷新设备",
              style: TextStyle(color: Colors.white),
            ),
            isLoading
                ? SizedBox(
                    width: 15,
                    height: 15,
                    child: CircularProgressIndicator(),
                  )
                : Text('')
          ],
        ),
      ),
    );
  }

  Widget _filterCheckBox() {
    return Container(
      height: 50,
      width: 150,
      margin: EdgeInsets.symmetric(horizontal: 15),
      color: Colors.lightGreen,
      child: Row(
        children: <Widget>[
          Checkbox(
              value: isFilterWebDevices,
              onChanged: (b) {
                setState(() {
                  isFilterWebDevices = b;
                });
              }),
          Text(
            "过滤web设备",
            style: TextStyle(color: Colors.white),
          )
        ],
      ),
    );
  }

  /// 拉取设备列表
  _fetchDeviceList() async {
    setState(() {
      isLoading = true;
    });
    try {
      var response = await NetUtils.get("/api/device/list");
      var pre = await DeviceManager.currentDevice();
      setState(() {
        list = SimpleDevice.parseDevices(response);
        if (list.isNotEmpty) {
          currentDevice = list.firstWhere((item) => item.deviceId == pre.deviceId,
              orElse: () => list[0]);
          String jsonString = jsonEncode(currentDevice.toJson());
          SpUtil.setString("current_device", jsonString);
          Logger.info("device list count : ${list.length}");
        } else {
          SpUtil.setString("current_device", "");
          Logger.warning("device list empty");
        }
      });
    } catch (e) {
      Logger.error("fetch device exceptioned, e: ${e.toString()}");
    } finally {
      setState(() {
        isLoading = false;
      });
    }
  }
}

/// 设备item
class DeviceItem extends StatelessWidget {
  final SimpleDevice device;
  final bool isSelectedOne;
  final DeviceItemClick click;

  const DeviceItem({Key key, this.device, this.click, this.isSelectedOne})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var color = isSelectedOne ? Colors.green : Colors.grey;
    var borderWidth = isSelectedOne ? 3.0 : 1.0;
    var logo = Icons.device_unknown;
    switch (device.platform) {
      case "android":
        logo = Icons.android;
        break;
      case "ios":
        logo = Icons.phone_iphone;
        break;
      case "web":
        logo = Icons.web;
        break;
    }

    return GestureDetector(
      onTap: () {
        click(device);
      },
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          ConstrainedBox(
            constraints:
                BoxConstraints(minWidth: 200, maxWidth: double.infinity),
            child: Container(
              alignment: Alignment.center,
              margin: EdgeInsets.symmetric(vertical: 10),
              decoration: BoxDecoration(
                  border: Border.all(color: color, width: borderWidth)),
              padding: EdgeInsets.symmetric(vertical: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Icon(
                    logo,
                    color: Colors.white,
                    size: 20,
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 10),
                    child: Text(
                      device.deviceName,
                      style: TextStyle(color: Colors.white, fontSize: 20),
                    ),
                  ),
                ],
              ),
            ),
          ),
          SizedBox(
            width: 50,
            height: 50,
            child: isSelectedOne
                ? Icon(
                    Icons.done,
                    color: Colors.green,
                  )
                : Text(''),
          )
        ],
      ),
    );
  }
}

typedef DeviceItemClick = void Function(SimpleDevice device);
